package com.gihon.sso.auth;

import java.util.Enumeration;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import com.gihon.common.auth.AuthConstans;
import com.gihon.common.auth.AuthUtils;
import com.gihon.common.properties.GihonCommonProperties;
import com.gihon.sso.entity.vo.UserInfo;
import com.gihon.sso.service.TokenService;

import lombok.extern.slf4j.Slf4j;


/**
 * token转UserInfo并存放于请求域
 */
@Order(1)
@Slf4j
@Component
public class AuthTokenInterceptor implements HandlerInterceptor {

    @Resource
    private TokenService tokenService;
    

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }
        String token = parseToken(request);
        //认证token是否合法
        if (StringUtils.isNotBlank(token)) {
        	request.setAttribute(AuthUtils.REQ_TOKEN, token);
        	UserInfo checkResult = tokenService.checkTokenInfo(token);
            if (checkResult!=null) {
            	AuthUtils.setUser(checkResult);
            }
        }
        return true;
    }

	protected String parseToken(HttpServletRequest request) {
		// first check the header...
		String token = parseHeaderToken(request);

		// bearer type allows a request parameter as well
		if (token == null) {
			token = request.getParameter(AuthConstans.ACCESS_TOKEN);
			if (token == null) {
			}
		}

		return token;
	}

	@SuppressWarnings("unchecked")
	protected String parseHeaderToken(HttpServletRequest request) {
		Enumeration<String> headers = request.getHeaders(AuthConstans.HEADE_TOKEN);
		while (headers.hasMoreElements()) { // typically there is only one (most servers enforce that)
			String value = headers.nextElement();
			if ((value.toLowerCase().startsWith(AuthConstans.BEARER_TYPE.toLowerCase()))) {
				String authHeaderValue = value.substring(AuthConstans.BEARER_TYPE.length()).trim();
				int commaIndex = authHeaderValue.indexOf(',');
				if (commaIndex > 0) {
					authHeaderValue = authHeaderValue.substring(0, commaIndex);
				}
				return authHeaderValue;
			} else {
				// todo: support additional authorization schemes for different token types,
				// e.g. "MAC" specified by
				// http://tools.ietf.org/html/draft-hammer-oauth-v2-mac-token
				String authHeaderValue = value.trim();
				int commaIndex = authHeaderValue.indexOf(',');
				if (commaIndex > 0) {
					authHeaderValue = authHeaderValue.substring(0, commaIndex);
				}
				return authHeaderValue;
			}
		}

		return null;
	}
}
