package com.gihon.sso.entity.vo;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.gihon.common.auth.AuthUser;
import com.gihon.common.entity.GihonUser;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 获取用户信息
 * 
 * @author baihe
 *
 */
@Data
public class UserInfo  implements AuthUser,UserDetails, Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -5957738185840681340L;

	@ApiModelProperty("ID")
	private Long id;

	@ApiModelProperty("姓名")
	private String nickName;

	@ApiModelProperty("登录账号")
	private String username;

	@JsonIgnore
	private String password;

	@JsonIgnore
	private String salt;// 用userAccount代替
	
	@ApiModelProperty("公司ID")
	private Long companyId;// 公司ID

	@ApiModelProperty(value = "是否是公司管理员")
	private Boolean companyAdmin = false;
	/**
	 * 是否有效
	 */
	private boolean enabled = true;

	/**
	 * 账号非过期
	 */
	private boolean accountNonExpired = true;

	/**
	 * 账号非锁定
	 */
	private boolean accountNonLocked = true;

	/**
	 * 非过期认证
	 */
	private boolean credentialsNonExpired = true;
	/**
	 * 用户具有的角色集合（数据库中没有与之对应的字段，用户登录后系统自动注入）
	 */
	private Collection<? extends GrantedAuthority> authorities;
	
	public UserInfo() {
		
	}
	public UserInfo(GihonUser user) {
		this.id = user.getId();
		this.username = user.getUsername();
		this.nickName = user.getNickName();
		this.password = user.getPassword();
		this.salt = user.getUsername();
		this.companyId = user.getCompanyId();
		this.companyAdmin = user.getCompanyAdmin();
	}
	@Override
	public List<String> getRoleList() {
		if(authorities==null) {
			return null;
		}
		return authorities.stream().map(g->g.getAuthority()).collect(Collectors.toList());
	}
	
	public void setRoleList(List<String> roleList) {
		if(roleList==null) {
			return;
		}
		
		authorities = roleList.stream().map(r->{return new SimpleGrantedAuthority(r);} ).collect(Collectors.toList());
	}
}
