package com.gihon.sso.security;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.preauth.PreAuthenticatedAuthenticationToken;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class GihonSecurityFilter extends OncePerRequestFilter {
	
	@Autowired
	private AuthenticationEntryPoint authenticationEntryPoint = new GihonAuthenticationEntryPoint();

	@Autowired
	private AuthenticationManager authenticationManager;
	
	@Override
	protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
			throws ServletException, IOException {
		try {

			String tokenValue = parseToken(request);
			if (tokenValue == null) {
				logger.debug("No token in request, will continue chain.");
			} else {
				PreAuthenticatedAuthenticationToken authentication = new PreAuthenticatedAuthenticationToken(tokenValue,"");
				authentication.setDetails(tokenValue);
				Authentication authResult = authenticationManager.authenticate(authentication);
				logger.debug("Authentication success: " + authResult);
				SecurityContextHolder.getContext().setAuthentication(authResult);
			}
		} catch (AuthenticationException failed) {
			SecurityContextHolder.clearContext();

			logger.debug("Authentication request failed: " + failed);

			authenticationEntryPoint.commence(request, response,
					new InsufficientAuthenticationException(failed.getMessage(), failed));

			return;
		}

		chain.doFilter(request, response);
	}

	protected String parseToken(HttpServletRequest request) {
		// first check the header...
		String token = parseHeaderToken(request);

		// bearer type allows a request parameter as well
		if (token == null) {
			token = request.getParameter(SecurityConstans.ACCESS_TOKEN);
			if (token == null) {
			}
		}

		return token;
	}

	@SuppressWarnings("unchecked")
	protected String parseHeaderToken(HttpServletRequest request) {
		Enumeration<String> headers = request.getHeaders(SecurityConstans.HEADE_TOKEN);
		while (headers.hasMoreElements()) { // typically there is only one (most servers enforce that)
			String value = headers.nextElement();
			if ((value.toLowerCase().startsWith(SecurityConstans.BEARER_TYPE.toLowerCase()))) {
				String authHeaderValue = value.substring(SecurityConstans.BEARER_TYPE.length()).trim();
				int commaIndex = authHeaderValue.indexOf(',');
				if (commaIndex > 0) {
					authHeaderValue = authHeaderValue.substring(0, commaIndex);
				}
				return authHeaderValue;
			} else {
				// todo: support additional authorization schemes for different token types,
				// e.g. "MAC" specified by
				// http://tools.ietf.org/html/draft-hammer-oauth-v2-mac-token
			}
		}

		return null;
	}
}
