package com.gihon.common.auth;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import lombok.extern.slf4j.Slf4j;

/**
 * 当前用户认证工具
 * 
 * @author edz
 *
 */
@Slf4j
public class AuthUtils {

	public final static String ROLE_SUPER_ADMIN = "superadmin";// 超级管理员
	public final static String ROLE_COMPANY_ADMIN = "companyadmin";// 公司管理员
	

	public final static String REQ_USER = "_req_authuser_";
	public final static String REQ_TOKEN = "_req_token_";
	public final static String REQ_BLANK = "_req_blank_";// 参数中传递的token

	public final static Long TOKEN_EXPIRE_TIME = 7200L;// 3个小时

	public static ServletRequestAttributes getRequestAttributes() {
		RequestAttributes r = RequestContextHolder.currentRequestAttributes();
		return (ServletRequestAttributes) r;
	}

	public static HttpServletRequest getRequest() {
		ServletRequestAttributes r = getRequestAttributes();
		return r==null?null:r.getRequest();
	}

	public static HttpServletResponse getResponse() {
		ServletRequestAttributes r = getRequestAttributes();
		return r==null?null:r.getResponse();
	}

	public static Long getUserId() {
		AuthUser user = getUser();
		if (user != null) {
			return user.getId();
		}
		return null;
	}

	public static AuthUser getUser() {
		HttpServletRequest request = getRequest();
		if(request == null) {
			return null;
		}
		AuthUser user = null;
		Object o = request.getAttribute(REQ_USER);
		if (o != null) {
			user = (AuthUser) o;
		}
		return user;
	}

	public static Long getCompanyId() {
		AuthUser user = getUser();
		if (user != null ) {
			return user.getCompanyId();
		}
		return null;
	}


	public static void setUser(AuthUser user) {
		getRequest().setAttribute(REQ_USER, user);
	}

}
